@@ -1,4 +1,6 @@
 """Provide methods to bootstrap a Home Assistant instance."""
+from __future__ import annotations
+
 import asyncio
 import contextlib
 from datetime import datetime
@@ -8,7 +10,7 @@ import os
 import sys
 import threading
 from time import monotonic
-from typing import TYPE_CHECKING, Any, Dict, Optional, Set
+from typing import TYPE_CHECKING, Any
 
 import voluptuous as vol
 import yarl
@@ -17,17 +19,20 @@ from homeassistant import config as conf_util, config_entries, core, loader
 from homeassistant.components import http
 from homeassistant.const import REQUIRED_NEXT_PYTHON_DATE, REQUIRED_NEXT_PYTHON_VER
 from homeassistant.exceptions import HomeAssistantError
+from homeassistant.helpers import area_registry, device_registry, entity_registry
+from homeassistant.helpers.dispatcher import async_dispatcher_send
 from homeassistant.helpers.typing import ConfigType
 from homeassistant.setup import (
     DATA_SETUP,
     DATA_SETUP_STARTED,
+    DATA_SETUP_TIME,
     async_set_domains_to_be_loaded,
     async_setup_component,
 )
 from homeassistant.util.async_ import gather_with_concurrency
+import homeassistant.util.dt as dt_util
 from homeassistant.util.logging import async_activate_log_queue_handler
 from homeassistant.util.package import async_get_user_site, is_virtual_env
-from homeassistant.util.yaml import clear_secret_cache
 
 if TYPE_CHECKING:
     from .runner import RuntimeConfig
@@ -40,6 +45,8 @@ ERROR_LOG_FILENAME = "home-assistant.log"
 DATA_LOGGING = "logging"
 
 LOG_SLOW_STARTUP_INTERVAL = 60
+SLOW_STARTUP_CHECK_INTERVAL = 1
+SIGNAL_BOOTSTRAP_INTEGRATONS = "bootstrap_integrations"
 
 STAGE_1_TIMEOUT = 120
 STAGE_2_TIMEOUT = 300
@@ -48,7 +55,7 @@ COOLDOWN_TIME = 60
 
 MAX_LOAD_CONCURRENTLY = 6
 
-DEBUGGER_INTEGRATIONS = {"debugpy", "ptvsd"}
+DEBUGGER_INTEGRATIONS = {"debugpy"}
 CORE_INTEGRATIONS = ("homeassistant", "persistent_notification")
 LOGGING_INTEGRATIONS = {
     # Set log levels
@@ -74,8 +81,8 @@ STAGE_1_INTEGRATIONS = {
 
 
 async def async_setup_hass(
-    runtime_config: "RuntimeConfig",
-) -> Optional[core.HomeAssistant]:
+    runtime_config: RuntimeConfig,
+) -> core.HomeAssistant | None:
     """Set up Home Assistant."""
     hass = core.HomeAssistant()
     hass.config.config_dir = runtime_config.config_dir
@@ -121,8 +128,6 @@ async def async_setup_hass(
             basic_setup_success = (
                 await async_from_config_dict(config_dict, hass) is not None
             )
-        finally:
-            clear_secret_cache()
 
     if config_dict is None:
         safe_mode = True
@@ -190,7 +195,7 @@ def open_hass_ui(hass: core.HomeAssistant) -> None:
 
 async def async_from_config_dict(
     config: ConfigType, hass: core.HomeAssistant
-) -> Optional[core.HomeAssistant]:
+) -> core.HomeAssistant | None:
     """Try to configure Home Assistant from a configuration dictionary.
 
     Dynamically loads required components and its dependencies.
@@ -257,8 +262,8 @@ async def async_from_config_dict(
 def async_enable_logging(
     hass: core.HomeAssistant,
     verbose: bool = False,
-    log_rotate_days: Optional[int] = None,
-    log_file: Optional[str] = None,
+    log_rotate_days: int | None = None,
+    log_file: str | None = None,
     log_no_color: bool = False,
 ) -> None:
     """Set up the logging.
@@ -307,12 +312,10 @@ def async_enable_logging(
     sys.excepthook = lambda *args: logging.getLogger(None).exception(
         "Uncaught exception", exc_info=args  # type: ignore
     )
-
-    if sys.version_info[:2] >= (3, 8):
-        threading.excepthook = lambda args: logging.getLogger(None).exception(
-            "Uncaught thread exception",
-            exc_info=(args.exc_type, args.exc_value, args.exc_traceback),
-        )
+    threading.excepthook = lambda args: logging.getLogger(None).exception(
+        "Uncaught thread exception",
+        exc_info=(args.exc_type, args.exc_value, args.exc_traceback),  # type: ignore[arg-type]
+    )
 
     # Log errors to a file if we have write access to file or config dir
     if log_file is None:
@@ -366,7 +369,7 @@ async def async_mount_local_lib_path(config_dir: str) -> str:
 
 
 @core.callback
-def _get_domains(hass: core.HomeAssistant, config: Dict[str, Any]) -> Set[str]:
+def _get_domains(hass: core.HomeAssistant, config: dict[str, Any]) -> set[str]:
     """Get domains of components to set up."""
     # Filter out the repeating and common config section [homeassistant]
     domains = {key.split(" ")[0] for key in config if key != core.DOMAIN}
@@ -382,35 +385,46 @@ def _get_domains(hass: core.HomeAssistant, config: Dict[str, Any]) -> Set[str]:
     return domains
 
 
-async def _async_log_pending_setups(
-    domains: Set[str], setup_started: Dict[str, datetime]
-) -> None:
+async def _async_watch_pending_setups(hass: core.HomeAssistant) -> None:
     """Periodic log of setups that are pending for longer than LOG_SLOW_STARTUP_INTERVAL."""
+    loop_count = 0
+    setup_started: dict[str, datetime] = hass.data[DATA_SETUP_STARTED]
+    previous_was_empty = True
     while True:
-        await asyncio.sleep(LOG_SLOW_STARTUP_INTERVAL)
-        remaining = [domain for domain in domains if domain in setup_started]
+        now = dt_util.utcnow()
+        remaining_with_setup_started = {
+            domain: (now - setup_started[domain]).total_seconds()
+            for domain in setup_started
+        }
+        _LOGGER.debug("Integration remaining: %s", remaining_with_setup_started)
+        if remaining_with_setup_started or not previous_was_empty:
+            async_dispatcher_send(
+                hass, SIGNAL_BOOTSTRAP_INTEGRATONS, remaining_with_setup_started
+            )
+        previous_was_empty = not remaining_with_setup_started
+        await asyncio.sleep(SLOW_STARTUP_CHECK_INTERVAL)
+        loop_count += SLOW_STARTUP_CHECK_INTERVAL
 
-        if remaining:
+        if loop_count >= LOG_SLOW_STARTUP_INTERVAL and setup_started:
             _LOGGER.warning(
                 "Waiting on integrations to complete setup: %s",
-                ", ".join(remaining),
+                ", ".join(setup_started),
             )
+            loop_count = 0
+        _LOGGER.debug("Running timeout Zones: %s", hass.timeout.zones)
 
 
 async def async_setup_multi_components(
     hass: core.HomeAssistant,
-    domains: Set[str],
-    config: Dict[str, Any],
-    setup_started: Dict[str, datetime],
+    domains: set[str],
+    config: dict[str, Any],
 ) -> None:
     """Set up multiple domains. Log on failure."""
     futures = {
         domain: hass.async_create_task(async_setup_component(hass, domain, config))
         for domain in domains
     }
-    log_task = asyncio.create_task(_async_log_pending_setups(domains, setup_started))
     await asyncio.wait(futures.values())
-    log_task.cancel()
     errors = [domain for domain in domains if futures[domain].exception()]
     for domain in errors:
         exception = futures[domain].exception()
@@ -423,15 +437,19 @@ async def async_setup_multi_components(
 
 
 async def _async_set_up_integrations(
-    hass: core.HomeAssistant, config: Dict[str, Any]
+    hass: core.HomeAssistant, config: dict[str, Any]
 ) -> None:
     """Set up all the integrations."""
-    setup_started = hass.data[DATA_SETUP_STARTED] = {}
+    hass.data[DATA_SETUP_STARTED] = {}
+    setup_time = hass.data[DATA_SETUP_TIME] = {}
+
+    watch_task = asyncio.create_task(_async_watch_pending_setups(hass))
+
     domains_to_setup = _get_domains(hass, config)
 
     # Resolve all dependencies so we know all integrations
     # that will have to be loaded and start rightaway
-    integration_cache: Dict[str, loader.Integration] = {}
+    integration_cache: dict[str, loader.Integration] = {}
     to_resolve = domains_to_setup
     while to_resolve:
         old_to_resolve = to_resolve
@@ -475,14 +493,14 @@ async def _async_set_up_integrations(
     # Load logging as soon as possible
     if logging_domains:
         _LOGGER.info("Setting up logging: %s", logging_domains)
-        await async_setup_multi_components(hass, logging_domains, config, setup_started)
+        await async_setup_multi_components(hass, logging_domains, config)
 
     # Start up debuggers. Start these first in case they want to wait.
     debuggers = domains_to_setup & DEBUGGER_INTEGRATIONS
 
     if debuggers:
         _LOGGER.debug("Setting up debuggers: %s", debuggers)
-        await async_setup_multi_components(hass, debuggers, config, setup_started)
+        await async_setup_multi_components(hass, debuggers, config)
 
     # calculate what components to setup in what stage
     stage_1_domains = set()
@@ -509,10 +527,12 @@ async def _async_set_up_integrations(
 
     stage_2_domains = domains_to_setup - logging_domains - debuggers - stage_1_domains
 
-    # Kick off loading the registries. They don't need to be awaited.
-    asyncio.create_task(hass.helpers.device_registry.async_get_registry())
-    asyncio.create_task(hass.helpers.entity_registry.async_get_registry())
-    asyncio.create_task(hass.helpers.area_registry.async_get_registry())
+    # Load the registries
+    await asyncio.gather(
+        device_registry.async_load(hass),
+        entity_registry.async_load(hass),
+        area_registry.async_load(hass),
+    )
 
     # Start setup
     if stage_1_domains:
@@ -521,9 +541,7 @@ async def _async_set_up_integrations(
             async with hass.timeout.async_timeout(
                 STAGE_1_TIMEOUT, cool_down=COOLDOWN_TIME
             ):
-                await async_setup_multi_components(
-                    hass, stage_1_domains, config, setup_started
-                )
+                await async_setup_multi_components(hass, stage_1_domains, config)
         except asyncio.TimeoutError:
             _LOGGER.warning("Setup timed out for stage 1 - moving forward")
 
@@ -536,12 +554,23 @@ async def _async_set_up_integrations(
             async with hass.timeout.async_timeout(
                 STAGE_2_TIMEOUT, cool_down=COOLDOWN_TIME
             ):
-                await async_setup_multi_components(
-                    hass, stage_2_domains, config, setup_started
-                )
+                await async_setup_multi_components(hass, stage_2_domains, config)
         except asyncio.TimeoutError:
             _LOGGER.warning("Setup timed out for stage 2 - moving forward")
 
+    watch_task.cancel()
+    async_dispatcher_send(hass, SIGNAL_BOOTSTRAP_INTEGRATONS, {})
+
+    _LOGGER.debug(
+        "Integration setup times: %s",
+        {
+            integration: timedelta.total_seconds()
+            for integration, timedelta in sorted(
+                setup_time.items(), key=lambda item: item[1].total_seconds()  # type: ignore
+            )
+        },
+    )
+
     # Wrap up startup
     _LOGGER.debug("Waiting for startup to wrap up")
     try:
