@@ -1,7 +1,10 @@
 """The Shelly integration."""
+from __future__ import annotations
+
 import asyncio
 from datetime import timedelta
 import logging
+from typing import Any, Final, cast
 
 import aioshelly
 import async_timeout
@@ -15,10 +18,11 @@ from homeassistant.const import (
     CONF_USERNAME,
     EVENT_HOMEASSISTANT_STOP,
 )
-from homeassistant.core import HomeAssistant, callback
+from homeassistant.core import Event, HomeAssistant, callback
 from homeassistant.exceptions import ConfigEntryNotReady
 from homeassistant.helpers import aiohttp_client, device_registry, update_coordinator
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.typing import ConfigType
 
 from .const import (
     AIOSHELLY_DEVICE_TIMEOUT_SEC,
@@ -43,19 +47,19 @@ from .const import (
 )
 from .utils import get_coap_context, get_device_name, get_device_sleep_period
 
-PLATFORMS = ["binary_sensor", "cover", "light", "sensor", "switch"]
-SLEEPING_PLATFORMS = ["binary_sensor", "sensor"]
-_LOGGER = logging.getLogger(__name__)
+PLATFORMS: Final = ["binary_sensor", "cover", "light", "sensor", "switch"]
+SLEEPING_PLATFORMS: Final = ["binary_sensor", "sensor"]
+_LOGGER: Final = logging.getLogger(__name__)
 
-COAP_SCHEMA = vol.Schema(
+COAP_SCHEMA: Final = vol.Schema(
     {
         vol.Optional(CONF_COAP_PORT, default=DEFAULT_COAP_PORT): cv.port,
     }
 )
-CONFIG_SCHEMA = vol.Schema({DOMAIN: COAP_SCHEMA}, extra=vol.ALLOW_EXTRA)
+CONFIG_SCHEMA: Final = vol.Schema({DOMAIN: COAP_SCHEMA}, extra=vol.ALLOW_EXTRA)
 
 
-async def async_setup(hass: HomeAssistant, config: dict):
+async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up the Shelly component."""
     hass.data[DOMAIN] = {DATA_CONFIG_ENTRY: {}}
 
@@ -66,7 +70,7 @@ async def async_setup(hass: HomeAssistant, config: dict):
     return True
 
 
-async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up Shelly from a config entry."""
     # The custom component for Shelly devices uses shelly domain as well as core
     # integration. If the user removes the custom component but doesn't remove the
@@ -113,7 +117,7 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
     sleep_period = entry.data.get("sleep_period")
 
     @callback
-    def _async_device_online(_):
+    def _async_device_online(_: Any) -> None:
         _LOGGER.debug("Device %s is online, resuming setup", entry.title)
         hass.data[DOMAIN][DATA_CONFIG_ENTRY][entry.entry_id][DEVICE] = None
 
@@ -153,7 +157,7 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
 
 async def async_device_setup(
     hass: HomeAssistant, entry: ConfigEntry, device: aioshelly.Device
-):
+) -> None:
     """Set up a device that is online."""
     device_wrapper = hass.data[DOMAIN][DATA_CONFIG_ENTRY][entry.entry_id][
         COAP
@@ -174,9 +178,11 @@ async def async_device_setup(
 class ShellyDeviceWrapper(update_coordinator.DataUpdateCoordinator):
     """Wrapper for a Shelly device with Home Assistant specific functions."""
 
-    def __init__(self, hass, entry, device: aioshelly.Device):
+    def __init__(
+        self, hass: HomeAssistant, entry: ConfigEntry, device: aioshelly.Device
+    ) -> None:
         """Initialize the Shelly device wrapper."""
-        self.device_id = None
+        self.device_id: str | None = None
         sleep_period = entry.data["sleep_period"]
 
         if sleep_period:
@@ -205,7 +211,7 @@ class ShellyDeviceWrapper(update_coordinator.DataUpdateCoordinator):
         hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, self._handle_ha_stop)
 
     @callback
-    def _async_device_updates_handler(self):
+    def _async_device_updates_handler(self) -> None:
         """Handle device updates."""
         if not self.device.initialized:
             return
@@ -258,7 +264,7 @@ class ShellyDeviceWrapper(update_coordinator.DataUpdateCoordinator):
                     self.name,
                 )
 
-    async def _async_update_data(self):
+    async def _async_update_data(self) -> None:
         """Fetch data."""
         if self.entry.data.get("sleep_period"):
             # Sleeping device, no point polling it, just mark it unavailable
@@ -267,21 +273,21 @@ class ShellyDeviceWrapper(update_coordinator.DataUpdateCoordinator):
         _LOGGER.debug("Polling Shelly Device - %s", self.name)
         try:
             async with async_timeout.timeout(POLLING_TIMEOUT_SEC):
-                return await self.device.update()
+                await self.device.update()
         except OSError as err:
             raise update_coordinator.UpdateFailed("Error fetching data") from err
 
     @property
-    def model(self):
+    def model(self) -> str:
         """Model of the device."""
-        return self.entry.data["model"]
+        return cast(str, self.entry.data["model"])
 
     @property
-    def mac(self):
+    def mac(self) -> str:
         """Mac address of the device."""
-        return self.entry.unique_id
+        return cast(str, self.entry.unique_id)
 
-    async def async_setup(self):
+    async def async_setup(self) -> None:
         """Set up the wrapper."""
         dev_reg = await device_registry.async_get_registry(self.hass)
         sw_version = self.device.settings["fw"] if self.device.initialized else ""
@@ -298,7 +304,7 @@ class ShellyDeviceWrapper(update_coordinator.DataUpdateCoordinator):
         self.device_id = entry.id
         self.device.subscribe_updates(self.async_set_updated_data)
 
-    def shutdown(self):
+    def shutdown(self) -> None:
         """Shutdown the wrapper."""
         if self.device:
             self.device.shutdown()
@@ -306,7 +312,7 @@ class ShellyDeviceWrapper(update_coordinator.DataUpdateCoordinator):
             self.device = None
 
     @callback
-    def _handle_ha_stop(self, _):
+    def _handle_ha_stop(self, _event: Event) -> None:
         """Handle Home Assistant stopping."""
         _LOGGER.debug("Stopping ShellyDeviceWrapper for %s", self.name)
         self.shutdown()
@@ -315,7 +321,7 @@ class ShellyDeviceWrapper(update_coordinator.DataUpdateCoordinator):
 class ShellyDeviceRestWrapper(update_coordinator.DataUpdateCoordinator):
     """Rest Wrapper for a Shelly device with Home Assistant specific functions."""
 
-    def __init__(self, hass, device: aioshelly.Device):
+    def __init__(self, hass: HomeAssistant, device: aioshelly.Device) -> None:
         """Initialize the Shelly device wrapper."""
         if (
             device.settings["device"]["type"]
@@ -335,22 +341,22 @@ class ShellyDeviceRestWrapper(update_coordinator.DataUpdateCoordinator):
         )
         self.device = device
 
-    async def _async_update_data(self):
+    async def _async_update_data(self) -> None:
         """Fetch data."""
         try:
             async with async_timeout.timeout(AIOSHELLY_DEVICE_TIMEOUT_SEC):
                 _LOGGER.debug("REST update for %s", self.name)
-                return await self.device.update_status()
+                await self.device.update_status()
         except OSError as err:
             raise update_coordinator.UpdateFailed("Error fetching data") from err
 
     @property
-    def mac(self):
+    def mac(self) -> str:
         """Mac address of the device."""
-        return self.device.settings["device"]["mac"]
+        return cast(str, self.device.settings["device"]["mac"])
 
 
-async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
+async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload a config entry."""
     device = hass.data[DOMAIN][DATA_CONFIG_ENTRY][entry.entry_id].get(DEVICE)
     if device is not None:
@@ -370,3 +376,21 @@ async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
         hass.data[DOMAIN][DATA_CONFIG_ENTRY].pop(entry.entry_id)
 
     return unload_ok
+
+
+def get_device_wrapper(
+    hass: HomeAssistant, device_id: str
+) -> ShellyDeviceWrapper | None:
+    """Get a Shelly device wrapper for the given device id."""
+    if not hass.data.get(DOMAIN):
+        return None
+
+    for config_entry in hass.data[DOMAIN][DATA_CONFIG_ENTRY]:
+        wrapper: ShellyDeviceWrapper | None = hass.data[DOMAIN][DATA_CONFIG_ENTRY][
+            config_entry
+        ].get(COAP)
+
+        if wrapper and wrapper.device_id == device_id:
+            return wrapper
+
+    return None
